// SCCSID "%Z% %W%  %I% %E% %U%"
/*
 * <N_OCO_COPYRIGHT>
 * Licensed Materials - Property of IBM
 * 
 * 5724-H72, 5655-R36, 5724-L26, 5655-L82     
 * 
 * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * <NOC_COPYRIGHT>
 */

package com.ibm.scenario.wmq.jms;

import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;

import com.ibm.msg.client.jms.JmsConnectionFactory;
import com.ibm.msg.client.jms.JmsDestination;

/**
 * A simple application that sends a responds to a message and then sends
 * a reply. Note: It is assumed that some other application will send the request
 * message and pick up the response
 * 
 * Application makes use of fixed literals, any customisations will require re-compilation of this
 * source file. Application assumes that the named queue is empty prior to a run.
 * 
 * It is assumed that the initial context contains the named definitions for connection factory and
 * destination objects. To create such an initial context (on the file system), launch the JMSAdmin
 * tool and issue the following commands:
 * 
 * InitCtx> DEF CF(myCF)
 * 
 * The command defines a connection factory object with the name 'myCF'. 
 *
 * Notes:
 * 
 * API type: JMS API (v1.1, unified domain)
 * 
 * Messaging domain: Point-to-point
 * 
 * Provider type: WebSphere MQ
 * 
 * Connection mode: Client connection
 * 
 * JNDI in use: Yes
 * 
 */
public class SimpleResponder {

  // System exit status value (assume unset value to be 1)
  private static int status = 1;

  /**
   * Main method
   * 
   * @param args
   */
  public static void main(String[] args) {

    // Variables
    Connection connection = null;
    ConnectionFactory connectionFactory = null;
    Session session = null;
    Destination destination = null;
    MessageProducer producer = null;
    MessageConsumer consumer = null;

    try {
      // Instantiate the initial context
      String contextFactory = "com.sun.jndi.fscontext.RefFSContextFactory";
      Hashtable environment = new Hashtable();
      environment.put(Context.INITIAL_CONTEXT_FACTORY, contextFactory);
      environment.put(Context.PROVIDER_URL, "file:/C:/JNDI-Directory");
      Context context = new InitialDirContext(environment);

      // Lookup the connection factory
      connectionFactory = (JmsConnectionFactory) context.lookup("myCF");
      System.out.println("> Connection factory located in JNDI.");

      // Lookup the destination
      destination = (JmsDestination) context.lookup("myQueue");
      System.out.println("> Destination located in JNDI.");

      // Create JMS objects
      System.out.println("> Creating connection to QueueManager.");
      connection = connectionFactory.createConnection();
      session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
      System.out.println("> Created connection.");

      // register an exception listener on the connection
      connection.setExceptionListener(new ExceptionListener() {

        public void onException(JMSException arg0) {
          System.out.println("Exception sent to exception listener");
          ExceptionHandling.handleException(arg0);
        }
      });

      // Create JMS consumer
      consumer = session.createConsumer(destination);

      // Start the connection
      connection.start();

      // Now, receive the reply
      System.out.println("\n> Waiting for message.");
      TextMessage receivedMessage = (TextMessage) consumer.receive(); // in ms or 15 seconds
      System.out.println("\n> Received Message ID=" + receivedMessage.getJMSMessageID() + " for '" + (receivedMessage).getText() + "'");

      // get the reply to destination
      Destination replyToDestination = receivedMessage.getJMSReplyTo();

      // create producer to reply to 
      producer = session.createProducer(replyToDestination);

      // create reply message
      String text = (receivedMessage).getText() + " - 15 tins in stock";
      System.out.println("\n> Sending Reply Message '" + text + "'");

      TextMessage message = session.createTextMessage();
      message.setText(text);

      // And, send the request
      producer.send(message);
      System.out.println("> Sent Message ID=" + message.getJMSMessageID());

    }
    catch (NamingException ne) {
      System.out.println("The initial context could not be instantiated, or the lookup failed.");
      ExceptionHandling.handleException(ne);
    }
    catch (JMSException jmsex) {
      ExceptionHandling.handleException(jmsex);
    }
    finally {
      System.out.println("\n> Closing connection to QueueManager.");
      if (producer != null) {
        try {
          producer.close();
        }
        catch (JMSException jmsex) {
          System.out.println("! Producer could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }
      if (consumer != null) {
        try {
          consumer.close();
        }
        catch (JMSException jmsex) {
          System.out.println("! Consumer could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }

      if (session != null) {
        try {
          session.close();
        }
        catch (JMSException jmsex) {
          System.out.println("! Session could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }

      if (connection != null) {
        try {
          connection.close();
        }
        catch (JMSException jmsex) {
          System.out.println("! Connection could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }
      System.out.println("> Closed connection.");
    }

    System.exit(status);
  } // end main()
}
